
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*  This code generated by: 
 *      Author : thomas
 *      Date   : Mon Sep 1 2003
 */



#ifndef CPPCODECOMMENT_H
#define CPPCODECOMMENT_H

#include <qstring.h>
#include "../codecomment.h"

/**
  * class CPPCodeComment
  * A CPP code comment. There is only a single styles of comments: 
  * these are simply started with double slash sequence and no terminating
  * characters
  */

class CPPCodeComment : virtual public CodeComment
{
	Q_OBJECT
public:

	// Constructors/Destructors
	//  


	/**
	 * Constructors
	 */
	CPPCodeComment ( CodeDocument * doc, const QString & text = "");

	/**
	 * Empty Destructor
	 */
	virtual ~CPPCodeComment ( );

	// Public attributes
	//  


	// other

	/**
	 * Save the XMI representation of this object
	 */
	virtual void saveToXMI ( QDomDocument & doc, QDomElement & root );

	/**
	 * @return	QString
	 */
	QString toString ( );


       /** UnFormat a long text string. Typically, this means removing
	 *  the indentaion (linePrefix) and/or newline chars from each line.
	 */
	virtual QString unformatText ( const QString & text, const QString & indent = "" );

	/** a special version here because we want to not only indent
	 * the new line, but to add the "//" sequence as well.
	 */
	virtual QString getNewEditorLine ( int amount );

protected:

private:

};

#endif // CPPCODECOMMENT_H
