
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*  This code generated by: 
 *      Author : thomas
 *      Date   : Fri Aug 31 2003
 */
 
#ifndef CPPHEADERACCESSORMETHOD_H
#define CPPHEADERACCESSORMETHOD_H

#include <qstring.h>

#include "../codeaccessormethod.h"
#include "cppcodeclassfield.h"

class CPPHeaderCodeAccessorMethod : public CodeAccessorMethod
{
	Q_OBJECT
public:

	// Constructors/Destructors
	//  

	/**
	 * Empty Constructor
	 */
	CPPHeaderCodeAccessorMethod ( CPPCodeClassField * field, CodeAccessorMethod::AccessorType type);

	/**
	 * Empty Destructor
	 */
	virtual ~CPPHeaderCodeAccessorMethod ( );

protected:

	virtual void updateMethodDeclaration();
	virtual void updateContent();

private:

};

#endif // CPPHEADERACCESSORMETHOD_H
