
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*  This code generated by: 
 *      Author : thomas
 *      Date   : Thu Oct 2 2003
 */
 
#ifndef JAVACODEACCESSORMETHOD_H
#define JAVACODEACCESSORMETHOD_H

#include <qstring.h>

#include "../codeaccessormethod.h"
#include "javacodeclassfield.h"

class JavaCodeAccessorMethod : public CodeAccessorMethod
{
	Q_OBJECT
public:

	// Constructors/Destructors
	//  

	/**
	 * Constructor
	 */
	JavaCodeAccessorMethod ( JavaCodeClassField * field, CodeAccessorMethod::AccessorType type);

	/**
	 * Empty Destructor
	 */
	virtual ~JavaCodeAccessorMethod ( );

protected:

        /** set attributes of the node that represents this class
         * in the XMI document.
         */
        virtual void setAttributesOnNode ( QDomDocument & doc, QDomElement & blockElement);

        /** set the class attributes of this object from
         * the passed element node.
         */
        virtual void setAttributesFromNode ( QDomElement & element);

	virtual void updateMethodDeclaration();
	virtual void updateContent();

private:

	void init (JavaCodeClassField * field);

};

#endif // JAVACODEACCESSORMETHOD_H
