
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*  This code generated by:
 *      Author : thomas
 *      Date   : Fri Jul 25 2003
 */

#ifndef JAVACODECLASSFIELDDECLARATIONBLOCK_H
#define JAVACODECLASSFIELDDECLARATIONBLOCK_H

#include <qstring.h>

#include "../codeclassfielddeclarationblock.h"

class JavaCodeClassFieldDeclarationBlock : public CodeClassFieldDeclarationBlock
{
	Q_OBJECT
public:

	// Constructors/Destructors
	//  

	/**
	 * Constructor
	 */
	JavaCodeClassFieldDeclarationBlock ( CodeClassField * parent );

	/**
	 * Empty Destructor
	 */
	virtual ~JavaCodeClassFieldDeclarationBlock ( );

protected:

	// this will be called by syncToParent whenever the parent object is "modified" 
       	void updateContent ( );

private:


};

#endif // JAVACODECLASSFIELDDECLARATIONBLOCK_H
