/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*  This code generated by:
 *      Author : thomas
 *      Date   : Wed Jul 30 2003
 */

#ifndef JAVACODEGENERATIONPOLICYPAGE_H
#define JAVACODEGENERATIONPOLICYPAGE_H

#include "../dialogs/codegenerationpolicypage.h"
#include "javacodegenerationformbase.h"

#include "javacodegenerationpolicy.h"

/**
 * @author Brian Thomas
 */

class JavaCodeGenerationPolicyPage : public CodeGenerationPolicyPage {
	Q_OBJECT
public:

	JavaCodeGenerationPolicyPage (QWidget *parent=0, const char *name=0, JavaCodeGenerationPolicy * policy = 0);

	virtual ~JavaCodeGenerationPolicyPage();

protected:

	JavaCodeGenerationFormBase * form;
	int commentTypeToInteger(JavaCodeGenerationPolicy::JavaCommentStyle type);

public slots:

	void apply();

};

#endif

