
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*  This code generated by: 
 *      Author : thomas
 *      Date   : Tue Jun 24 2003
 */



#ifndef JAVAANTCODEDOCUMENT_H
#define JAVAANTCODEDOCUMENT_H

#include <qstring.h>
#include <qdom.h>

class JavaCodeGenerator;
#include "../codedocument.h"

/**
  * class JavaANTCodeDocument
  * Represents 
  */

class JavaANTCodeDocument : public CodeDocument
{
	Q_OBJECT
public:

	// Constructors/Destructors
	//  

	/**
	 * Constructor
	 */
	JavaANTCodeDocument ( JavaCodeGenerator * gen);

	/**
	 * Empty Destructor
	 */
	virtual ~JavaANTCodeDocument ( );

	QString getPath ( );

	void updateContent();

       /**
	 * Save the XMI representation of this object
	 */
	virtual void saveToXMI ( QDomDocument & doc, QDomElement & root );

	/**
	 * load params from the appropriate XMI element node.
	 */
	virtual void loadFromXMI ( QDomElement & root );

	/**
	 * create a new HierarchicalCodeBlock object belonging to this CodeDocument.
	 * @return      HierarchicalCodeBlock
	 */
	virtual HierarchicalCodeBlock * newHierarchicalCodeBlock ( );

	/**
	 * create a new CodeBlockWithComments object belonging to this CodeDocument.
	 * @return      CodeBlockWithComments
	 */
//	virtual CodeBlockWithComments * newCodeBlockWithComments ( );

	// create a code comment for this document
	virtual CodeComment * newCodeComment();

protected:

       /** set attributes of the node that represents this class
	 * in the XMI document.
	 */
	virtual void setAttributesOnNode ( QDomDocument & doc, QDomElement & blockElement);

	/** set the class attributes of this object from
	 * the passed element node.
	 */
	virtual void setAttributesFromNode ( QDomElement & element);

       /**
	 * need to overwrite this for java since we need to pick up the
	 * xml declaration blocks.
	 */
	virtual void loadChildTextBlocksFromNode ( QDomElement & root);

private:



};

#endif // JAVAANTCODEDOCUMENT_H
